# Miscellaneous module

import paramiko
from pySSHAdmin.utilities import _check_connection


def execute_command(host, user, passw, command, suppress=False):
    """Execute command via ssh

        Parameters:

        host (String): hostname or IP of remote server
        user (String): valid username from remote server
        passwd (String): password associated with username
        suppress (boolean): suppresses exit code recognition print (default False

        Returns:

        int: exit code 0 (valid, executed) or 127 wten command was not found or -1 in case of error
    """
    # Check connection first
    if _check_connection(host=host, user=user, passw=passw, suppress=True) != 0:
        if not suppress:
            print("Connection to host failed!")
        return -1

    ssh = paramiko.SSHClient()
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    ssh.connect(hostname=host, port=22, username=user, password=passw)
    _ssh_stdin, _ssh_stdout, _ssh_stderr = ssh.exec_command(command + "\n", get_pty=True)
    exit_status = _ssh_stdout.channel.recv_exit_status()
    response = ""
    if exit_status == 0:
        response = "Done!"
    elif exit_status == 127:
        response = "Command was not found!"
    else:
        response = "Error {}".format(exit_status)
        exit_status = -1

    if not suppress:
        print(response)

    ssh.close()

    return exit_status
