# Services module
import paramiko
from pySSHAdmin.utilities import _check_connection


def stop_service(host, user, passw, service_name, suppress=False):
    """Stop service on remote

        Parameters:

        host (String): hostname or IP of remote server
        user (String): valid username from remote server
        passwd (String): password associated with username
        service_name (String): service name to stop
        suppress (boolean): suppresses exit code recognition print (default False)

        Returns:

        int: exit code 0 (success) or -1 in case of error
    """
    # Check connection first
    if _check_connection(host=host, user=user, passw=passw, suppress=True) != 0:
        if not suppress: print("Connection to host failed!")
        return -1

    exit_status = -1
    response = ""

    try:
        ssh = paramiko.SSHClient()
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ssh.connect(hostname=host, port=22, username=user, password=passw)
        _ssh_stdin, _ssh_stdout, _ssh_stderr = ssh.exec_command("systemctl stop {}".format(service_name))
        exit_status = _ssh_stdout.channel.recv_exit_status()

    except paramiko.SSHException as exception:
        if not suppress: print(exception)
        return -1

    finally:
        ssh.close()

    if exit_status == 0:
        response = "Done"
    else:
        response = "Ooops... something went wrong, check status manually"

    if not suppress: print(response)
    return exit_status


def start_service(host, user, passw, service_name, suppress=False):
    """Start service on remote

        Parameters:

        host (String): hostname or IP of remote server
        user (String): valid username from remote server
        passwd (String): password associated with username
        service_name (String): service name to start
        suppress (boolean): suppresses exit code recognition print (default False)

        Returns:

        int: exit code 0 (success) or -1 in case of error
    """
    # Check connection first
    if _check_connection(host=host, user=user, passw=passw, suppress=True) != 0:
        if not suppress: print("Connection to host failed!")
        return -1

    exit_status = -1
    response = ""

    try:
        ssh = paramiko.SSHClient()
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ssh.connect(hostname=host, port=22, username=user, password=passw)
        _ssh_stdin, _ssh_stdout, _ssh_stderr = ssh.exec_command("systemctl start {}".format(service_name))
        exit_status = _ssh_stdout.channel.recv_exit_status()

    except paramiko.SSHException as exception:
        if not suppress: print(exception)
        return -1

    finally:
        ssh.close()

    if exit_status == 0:
        response = "Done"
    else:
        response = "Ooops... something went wrong, check status manually"

    if not suppress: print(response)
    return exit_status


def restart_service(host, user, passw, service_name, suppress=False):
    """Restart service on remote

        Parameters:

        host (String): hostname or IP of remote server
        user (String): valid username from remote server
        passwd (String): password associated with username
        service_name (String): service name to restart
        suppress (boolean): suppresses exit code recognition print (default False)

        Returns:

        int: exit code 0 (success) or -1 in case of error
    """
    # Check connection first
    if _check_connection(host=host, user=user, passw=passw, suppress=True) != 0:
        if not suppress: print("Connection to host failed!")
        return -1

    exit_status = -1
    response = ""

    try:
        ssh = paramiko.SSHClient()
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ssh.connect(hostname=host, port=22, username=user, password=passw)
        _ssh_stdin, _ssh_stdout, _ssh_stderr = ssh.exec_command("systemctl restart {}".format(service_name))
        exit_status = _ssh_stdout.channel.recv_exit_status()

    except paramiko.SSHException as exception:
        if not suppress: print(exception)
        return -1

    finally:
        ssh.close()

    if exit_status == 0:
        response = "Done"
    else:
        response = "Ooops... something went wrong, check status manually"

    if not suppress: print(response)
    return exit_status


def enable_service(host, user, passw, service_name, suppress=False):
    """Enable service on remote

        Parameters:

        host (String): hostname or IP of remote server
        user (String): valid username from remote server
        passwd (String): password associated with username
        service_name (String): service name to enable
        suppress (boolean): suppresses exit code recognition print (default False)

        Returns:

        int: exit code 0 (success) or -1 in case of error
    """
    # Check connection first
    if _check_connection(host=host, user=user, passw=passw, suppress=True) != 0:
        if not suppress: print("Connection to host failed!")
        return -1

    exit_status = -1
    response = ""

    try:
        ssh = paramiko.SSHClient()
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ssh.connect(hostname=host, port=22, username=user, password=passw)
        _ssh_stdin, _ssh_stdout, _ssh_stderr = ssh.exec_command("systemctl enable {}".format(service_name))
        exit_status = _ssh_stdout.channel.recv_exit_status()

    except paramiko.SSHException as exception:
        if not suppress: print(exception)
        return -1

    finally:
        ssh.close()

    if exit_status == 0:
        response = "Done"
    else:
        response = "Ooops... something went wrong, check status manually"

    if not suppress: print(response)
    return exit_status


def disable_service(host, user, passw, service_name, suppress=False):
    """Disable service on remote

        Parameters:

        host (String): hostname or IP of remote server
        user (String): valid username from remote server
        passwd (String): password associated with username
        service_name (String): service name to disable
        suppress (boolean): suppresses exit code recognition print (default False)

        Returns:

        int: exit code 0 (success) or -1 in case of error
    """
    # Check connection first
    if _check_connection(host=host, user=user, passw=passw, suppress=True) != 0:
        if not suppress: print("Connection to host failed!")
        return -1

    exit_status = -1
    response = ""

    try:
        ssh = paramiko.SSHClient()
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ssh.connect(hostname=host, port=22, username=user, password=passw)
        _ssh_stdin, _ssh_stdout, _ssh_stderr = ssh.exec_command("systemctl disable {}".format(service_name))
        exit_status = _ssh_stdout.channel.recv_exit_status()

    except paramiko.SSHException as exception:
        if not suppress: print(exception)
        return -1

    finally:
        ssh.close()

    if exit_status == 0:
        response = "Done"
    else:
        response = "Ooops... something went wrong, check status manually"

    if not suppress: print(response)
    return exit_status


def service_status(host, user, passw, service_name, suppress=False):
    """Get service status from remote

        Parameters:

        host (String): hostname or IP of remote server
        user (String): valid username from remote server
        passwd (String): password associated with username
        service_name (String): service name to check status
        suppress (boolean): suppresses exit code recognition print (default False)

        Returns:

        int: exit code 0 (success) or -1 in case of error
    """
    # Check connection first
    if _check_connection(host=host, user=user, passw=passw, suppress=True) != 0:
        if not suppress: print("Connection to host failed!")
        return -1

    exit_status = -1
    response = ""

    try:
        ssh = paramiko.SSHClient()
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ssh.connect(hostname=host, port=22, username=user, password=passw)
        _ssh_stdin, _ssh_stdout, _ssh_stderr = ssh.exec_command("systemctl status {}".format(service_name))
        exit_status = _ssh_stdout.channel.recv_exit_status()

    except paramiko.SSHException as exception:
        if not suppress: print(exception)
        return -1

    finally:
        ssh.close()

    if exit_status == 0:
        response = "Done"
    else:
        response = "Ooops... something went wrong, or serive is not OK"

    if not suppress: print(response)
    return exit_status
